@@ -1,4 +1,6 @@
 #!/usr/bin/env perl
+# ABSTRACT: the asynchronous, non-blocking shadowsocks client and server. 
+# KEYWORDS: socks shadowsocks rfc1928 Great Firewall Internet censorship in China
 #
 # Modify the config.json sample, save it to config.json. Run ssserver.pl on
 # server, run ssclient.pl on localhost. Configure your browser to use socks 5
@@ -22,14 +24,14 @@ sub main::HELP_MESSAGE()
     print("\t-s SERVER_ADDR\tYour server host name or ip address\n");
     print("\t-p SERVER_PORT\tYour server port\n");
     print("\t-k PASSWORD\tYour top-secret password \n");
-    print("\t-m METHOD\t\tYour favarite encryption method, pick one of the methods below:\n");
-    print("\t\taes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb\n");
-    print("\t\taes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb\n");
-    print("\t\taes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb\n");
-    print("\t\tcamellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb\n");
-    print("\t\tcamellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb\n");
-    print("\t\tcamellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb\n");
-    print("\t\tchacha20-ietf rc4-md5 rc4-sha rc6\n");
+    print("\t-m METHOD\tYour favarite encryption method, pick one of the methods below:\n");
+    print("\taes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb\n");
+    print("\taes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb\n");
+    print("\taes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb\n");
+    print("\tcamellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb\n");
+    print("\tcamellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb\n");
+    print("\tcamellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb\n");
+    print("\tchacha20-ietf rc4-md5 rc4-sha rc6\n");
     printf("\t-c CONFIG\tName of your config.json file with full path\n");
     print("\t-h, --help\tDisplay help information\n");
     print("\t-v, --version\tDisplay version information\n");
@@ -46,9 +48,6 @@ main::HELP_MESSAGE() unless ($ARGV[0]);
 my %options=();
 getopts('hc:k:m:p:s:v',\%options);
 
-main::HELP_MESSAGE() if $options{h};
-main::VERSION_MESSAGE() if $options{v};
-
 if (defined($options{h}))
 {
     main::HELP_MESSAGE();
@@ -106,6 +105,27 @@ Version 0.8.1
 
 Shadowsocks protocol server.
 
+Usage: ssserver.pl -s SERVER_ADDR -p SERVER_PORT [-b LOCAL_ADDR]
+    -l LOCAL_PORT -k PASSWORD -m METHOD [-c CONFIG]
+
+	Pleae note that all optional arguments must be in lower case.
+
+	-s SERVER_ADDR	Your server host name or ip address
+	-p SERVER_PORT	Your server port
+	-k PASSWORD	Your top-secret password 
+	-m METHOD	Your favarite encryption method, pick one of the methods below:
+	aes-128-cbc aes-128-cfb aes-128-ctr aes-128-ofb
+	aes-192-cbc aes-192-cfb aes-192-ctr aes-192-ofb
+	aes-256-cbc aes-256-cfb aes-256-ctr aes-256-ofb
+	camellia-128-cbc camellia-128-cfb camellia-128-ctr camellia-128-ofb
+	camellia-192-cbc camellia-192-cfb camellia-192-ctr camellia-192-ofb
+	camellia-256-cbc camellia-256-cfb camellia-256-ctr camellia-256-ofb
+	chacha20-ietf rc4-md5 rc4-sha rc6
+	-c CONFIG	Name of your config.json file with full path
+	-h, --help	Display help information
+	-v, --version	Display version information
+
+
 =head1 DESCRIPTION
 
 Net::Shadowsocks is a Perl implementation of the shadowsocks (Chinese: 影梭)
