@@ -1,6 +1,7 @@
 package Net::Shadowsocks::Client;
 
     # ABSTRACT: Shadowsocks protocol client module.
+    # KEYWORDS: socks shadowsocks rfc1928 Great Firewall Internet censorship in China
     use 5.006;
     use strict;
     use warnings;
@@ -17,7 +18,7 @@ package Net::Shadowsocks::Client;
     use Net::Shadowsocks qw( _get_algorithm _get_mode _EVP_BytesToKey _initialize_cipher);
     use Socket qw(IPPROTO_TCP MSG_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
 
-    our $VERSION = '0.8.1';
+    our $VERSION = '0.8.2';
     $AnyEvent::Log::FILTER->level ("info");
     sub new($$$$$$) 
     {
@@ -63,7 +64,7 @@ package Net::Shadowsocks::Client;
                     on_eof    => sub 
                     {
                         my $chandle = shift;
-                        AE::log info => "Client: Done.";
+                        #AE::log info => "Client: Done.";
                         $chandle->destroy();
                     },
                     on_error => sub 
@@ -89,23 +90,28 @@ package Net::Shadowsocks::Client;
 
                                 if ( $ver != SOCKS5_VER )
                                  {
-                                    $! = ESOCKSPROTO;
-                                    $SOCKS_ERROR->set( ISS_BAD_VERSION, $@ ="Socks version should be 5, $ver recieved");
-                                    return;
+                                     AE::log error => "Socks version should be 5, $ver recieved";
+                                     $chandle->destroy();
+                                    #return;
                                 }
-                                my $nmethods = ord( substr( $client_buffer, 1, 1 ) );
-
-                                if ( $nmethods == 0 ) 
+                                else
                                 {
-                                    $! = ESOCKSPROTO;
-                                    $SOCKS_ERROR->set( AUTHMECH_INVALID, $@ = "No auth methods sent" );
-                                    return;
+                                    my $nmethods = ord( substr( $client_buffer, 1, 1 ) );
+
+                                    if ( $nmethods == 0 ) 
+                                    {
+                                        AE::log error =>  "No authentication methods sent" ;
+                                        $chandle->destroy();
+                                        #return;
+                                    }
+                                    else
+                                    {
+                                        my @methods = unpack( 'C' x $nmethods, substr( $client_buffer, 2 ) );
+
+                                        $clienthandler->push_write(pack( 'CC', SOCKS5_VER, 0 ) );
+                                        $stage = 1;
+                                    }
                                 }
-                                my @methods = unpack( 'C' x $nmethods, substr( $client_buffer, 2 ) );
-
-                                $clienthandler->push_write(pack( 'CC', SOCKS5_VER, 0 ) );
-                                $stage = 1;
-
                             }
                             elsif ( $stage == 1 ) 
                             {
@@ -120,60 +126,64 @@ package Net::Shadowsocks::Client;
                                           $chandle->destroy();
                                         #return;
                                     }
-                                    $addr_to_send = substr( $client_buffer, 3 );
-                                    if ( !defined( $self->{local_address} ) ) 
+                                    else
                                     {
-                                        $self->{local_address} = 'localhost';
-                                    }
-                                    my $hlen = length( $self->{local_address} );
-                                    $clienthandler->push_write(pack( 'CCCC',SOCKS5_VER, 0,0,          ADDR_DOMAINNAME )
+                                        $addr_to_send = substr( $client_buffer, 3 );
+                                        if ( !defined( $self->{local_address} ) ) 
+                                        {
+                                             $self->{local_address} = 'localhost';
+                                        }
+                                        my $hlen = length( $self->{local_address} );
+                                        $clienthandler->push_write(pack( 'CCCC',SOCKS5_VER, 0,0,          ADDR_DOMAINNAME )
                                           . pack( 'C', $hlen )
                                           . $self->{local_address}
                                           . pack( 'n', $self->{local_port} )
-                                    );
-                                    $stage = 4;
+                                        );
+                                        $stage = 4;
 
-                                    $remotehandler = AnyEvent::Handle->new
-                                    (
-                                        autocork  => 1,
-                                        keepalive => 1,
-                                        no_delay  => 1,
-                                        connect   => [$self->{server},$self->{server_port}],
-                                        on_connect => sub 
-                                        {
-                                            my ($chandle,  $peerhost,$peerport, $retry) = @_;
-
-                                            unless ($chandle) 
+                                        $remotehandler = AnyEvent::Handle->new
+                                        (
+                                            autocork  => 1,
+                                            keepalive => 1,
+                                            no_delay  => 1,
+                                            connect   => [$self->{server},$self->{server_port}],
+                                            on_connect => sub 
                                             {
-                                                AE::log info =>"couldn't connect: $!";
-                                                return;
-                                            }
-                                        },
-                                        on_eof => sub 
-                                        {
-                                            my $rhandle = shift;
-                                            #AE::log info => "Remote: Done.";
-                                            $rhandle->destroy();
-                                            $mode = 0;
-                                        },
-                                        on_error => sub 
-                                        {
-                                            my ( $rhandle, $fatal, $msg ) = @_;
-                                            AE::log error => $msg;
-                                            $rhandle->destroy();
-                                            $mode = 0;
-                                        },
-                                        on_prepare => sub
-                                        {
-                                            my $phandle = shift;
-                                            setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
-                                        },
-                                        on_read => sub 
-                                        {
-                                            my $incomingdata = $remotehandler->rbuf();
-                                            my $decrypteddata;
-                                            unless ( defined($server_iv) ) 
-                                             {
+                                                $stage = 5;
+                                            },
+                                            on_connect_error => sub
+                                            {
+                                                 my ($chandle,  $msg) = @_;
+                                                    AE::log error => $msg;
+                                                    $chandle->destroy();
+                                                    $mode = 0;
+                                                    #return;
+                                            },
+                                            on_eof => sub 
+                                            {
+                                                 my $rhandle = shift;
+                                                #AE::log info => "Remote: Done.";
+                                                $rhandle->destroy();
+                                                $mode = 0;
+                                            },
+                                            on_error => sub 
+                                            {
+                                                my ( $rhandle, $fatal, $msg ) = @_;
+                                                AE::log error => $msg;
+                                                $rhandle->destroy();
+                                                $mode = 0;
+                                            },
+                                            on_prepare => sub
+                                            {
+                                                my $phandle = shift;
+                                                setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                            },
+                                            on_read => sub 
+                                            {
+                                                my $incomingdata = $remotehandler->rbuf();
+                                                my $decrypteddata;
+                                                unless ( defined($server_iv) ) 
+                                                {
                                                     $server_iv = substr( $incomingdata, 0,length($iv) );
                                                     $incomingdata = substr($incomingdata,length($iv));
                                                     if ( !defined($self->{method}) or $self->{method} eq'rc4-md5' )
@@ -202,40 +212,40 @@ package Net::Shadowsocks::Client;
                                                     {
                                                             Mcrypt::mcrypt_init($decryptor,$key,$server_iv );
                                                     }
-                                              }
-                                              if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
-                                              {
+                                                }
+                                                if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
+                                                {
                                                     $decrypteddata = $decryptor->decrypt($incomingdata,$key,$server_iv);
-                                              }
+                                                }
 
-                                              elsif($self->{method} eq 'chacha20-ietf')
-                                              {
-                                                        $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$server_iv,$key);
-                                              }
+                                                elsif($self->{method} eq 'chacha20-ietf')
+                                                {
+                                                    $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$server_iv,$key);
+                                                }
 
-                                              else
-                                              {
+                                                else
+                                                {
                                                     $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
-                                              }
-                                              $clienthandler->push_write($decrypteddata);
-                                              $remotehandler->{rbuf} = '';
-                                         },
-                                     );
-                                    $stage = 5;
+                                                }
+                                                $clienthandler->push_write($decrypteddata);
+                                                $remotehandler->{rbuf} = '';
+                                             },
+                                         );
+                                    }
                                 }
                                 elsif ( $cmd == CMD_BIND ) 
                                 {
                                     carp 'BIND Request not supported';
                                     $stage = 0;
                                     $clienthandler->destroy();
-                                    return;
+                                    #return;
                                 }
                                 elsif ( $cmd == CMD_UDPASSOC ) 
                                 {
                                     carp 'UDP ASSOCIATE request not implemented';
                                     $stage = 0;
                                     $clienthandler->destroy();
-                                    return;
+                                    #return;
                                 }
 
                                 else 
@@ -243,7 +253,7 @@ package Net::Shadowsocks::Client;
                                     carp 'Unknown command';
                                     $stage = 0;
                                     $clienthandler->destroy();
-                                    return;
+                                    #return;
                                 }
                             }
 
@@ -251,24 +261,23 @@ package Net::Shadowsocks::Client;
                             {
                                 my $plaindata = $client_buffer;
                                 my $encrypteddata;
-                                         if ( $addr_to_send ne '' ) 
-                                         {
-                                             $plaindata = $addr_to_send . $client_buffer;
-                                            $addr_to_send = '';
-                                     }
-                                      if ( $self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/) 
-                                    {
-                                                                    
-                                          $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv);
-                                    }
-                                     elsif($self->{method} eq 'chacha20-ietf')
-                                                    {
-                                                        $encrypteddata = $encryptor->chacha20_ietf_xor($plaindata,$nonce,$key);
-                                                        }
-                                    else
-                                    {
+                                if ( $addr_to_send ne '' ) 
+                                {
+                                    $plaindata = $addr_to_send . $client_buffer;
+                                    $addr_to_send = '';
+                                }
+                                if ( $self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/) 
+                                {                                
+                                    $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv);
+                                }
+                                elsif($self->{method} eq 'chacha20-ietf')
+                                {
+                                     $encrypteddata = $encryptor->chacha20_ietf_xor($plaindata,$nonce,$key);
+                                }
+                                else
+                                {
                                      $encrypteddata = Mcrypt::mcrypt_encrypt($encryptor,$plaindata);
-                                 }
+                                }
                                 
                                 my $datatosend;
                                 if ( $mode == 0  ) 
@@ -282,7 +291,8 @@ package Net::Shadowsocks::Client;
                                     }
                                     $mode = 1;
                                 }
-                                else {
+                                else 
+                                {
                                     $datatosend = $encrypteddata;
                                 }
 
@@ -297,7 +307,7 @@ package Net::Shadowsocks::Client;
             {
                 my ($fh) = shift;
                 setsockopt($fh, SOL_SOCKET, SO_REUSEADDR, 1);
-                }
+            }
         );
         my $cv = AE::cv;
         $cv->recv();
@@ -317,7 +327,7 @@ Net::Shadowsocks::Client
 
 =head1 VERSION
 
-Version 0.8.1
+Version 0.8.2
 
 =head1 SYNOPSIS
 
