@@ -1,6 +1,7 @@
  package Net::Shadowsocks::Server;
 
     # ABSTRACT: Shadowsocks protocol server module.
+    # KEYWORDS: socks shadowsocks rfc1928 Great Firewall Internet censorship in China
     use 5.006;
     use strict;
     use warnings;
@@ -16,7 +17,7 @@
     use Net::Shadowsocks qw(_get_algorithm _get_mode _EVP_BytesToKey _initialize_cipher);
     use Socket qw(IPPROTO_TCP TCP_FASTOPEN SOL_SOCKET SO_REUSEPORT SO_REUSEADDR);
 
-    our $VERSION = '0.8.1';
+    our $VERSION = '0.8.2';
     
     $AnyEvent::Log::FILTER->level ("info");
     
@@ -59,7 +60,7 @@
                     on_eof    => sub 
                     {
                         my $chandler = shift;
-                        AE::log info => "Client: Done.";
+                        #AE::log info => "Client: Done.";
                         $chandler->destroy();
                     },
                     on_error => sub 
@@ -100,165 +101,165 @@
                                         #$md->add($client_iv);
                                         my $decrypt_rc4_key = substr($sha->digest(),0,16);
                                         Mcrypt::mcrypt_init($decryptor, $decrypt_rc4_key, '' );
-                                        }
-                                        elsif($self->{method} ne 'rc6' and $self->{method} ne 'chacha20-ietf' and  $self->{method} !~ /cfb$/ and $self-> {method} !~ /ctr$/ and $self->{method} !~ /^camellia/ )
-                                        {
+                                    }
+                                    elsif($self->{method} ne 'rc6' and $self->{method} ne 'chacha20-ietf' and  $self->{method} !~ /cfb$/ and $self-> {method} !~ /ctr$/ and $self->{method} !~ /^camellia/ )
+                                    {
                                             Mcrypt::mcrypt_init($decryptor, $key, $client_iv );
-                                            }
-                                            else
-                                            {
-                                                }                                    
+                                    }
+                                    else
+                                    {
+                                    
+                                    }                                    
                                     if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
                                     {
-                                    $decrypteddata = $decryptor->decrypt($incomingdata,$key,$client_iv);
-                                        }
-                                           elsif($self->{method} eq 'chacha20-ietf')
-                                       {
+                                        $decrypteddata = $decryptor->decrypt($incomingdata,$key,$client_iv);
+                                    }
+                                    elsif($self->{method} eq 'chacha20-ietf')
+                                    {
                                            #$decrypteddata = $decryptor->process($incomingdata);
-                                           $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$client_iv,$key);
-                                       }
-                                        else 
+                                       $decrypteddata = $decryptor->chacha20_ietf_xor($incomingdata,$client_iv,$key);
+                                    }
+                                    else 
                                     {
-                                    $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
-                                     }                                
-                                my $addrtype = ord( substr( $decrypteddata, 0, 1 ) );
-                                if (    $addrtype != 1 and $addrtype != 3 and $addrtype != 4 )
-                                {
-                                    carp ' addrtype not supported ';
-                                    return;
-                                }
-                                my $dest_addr;
-                                my $dest_port;
+                                        $decrypteddata = Mcrypt::mcrypt_decrypt($decryptor, $incomingdata );
+                                    }                                
+                                    my $addrtype = ord( substr( $decrypteddata, 0, 1 ) );
+                                    if (    $addrtype != 1 and $addrtype != 3 and $addrtype != 4 )
+                                    {
+                                        AE::log error => "Invalid address type";
+                                        #return;
+                                    }
+                                    else
+                                    {
+                                        my $dest_addr;
+                                        my $dest_port;
 
-                                if ( $addrtype == 1 ) 
-                                {
-                                    if ( length($decrypteddata) >= 7 ) 
-                                    {
-                                        $dest_addr = format_address(substr( $decrypteddata, 1, 4 ) );
-                                        $dest_port = unpack( ' n ',substr( $decrypteddata, 5, 2 ) );
-                                        $decrypteddata = substr( $decrypteddata, 7 );
-                                    }
-                                    else 
-                                    {
-                                        last;
-                                    }
-                                }
-                                elsif ( $addrtype == 3 ) 
-                                {
-                                    if ( length($decrypteddata) > 2 ) 
-                                    {
-                                        my $addr_len = ord( substr( $decrypteddata, 1, 1 ) );
-                                        if (length($decrypteddata) >= 2 + $addr_len )
+                                        if ( $addrtype == 1 ) 
                                         {
-                                            $dest_addr = substr( $decrypteddata, 2,$addr_len );
-                                            $dest_port = unpack('n',substr($decrypteddata,2 + $addr_len,2));
-                                            $decrypteddata = substr( $decrypteddata,4 + $addr_len );
+                                            if ( length($decrypteddata) >= 7 ) 
+                                            {
+                                                $dest_addr = format_address(substr( $decrypteddata, 1, 4 ) );
+                                                $dest_port = unpack( ' n ',substr( $decrypteddata, 5, 2 ) );
+                                                $decrypteddata = substr( $decrypteddata, 7 );
+                                            }
+                                            else 
+                                            {
+                                                return;
+                                            }
                                         }
-                                        else 
+                                        elsif ( $addrtype == 3 ) 
                                         {
-                                            last;
+                                            if ( length($decrypteddata) > 2 ) 
+                                            {
+                                                my $addr_len = ord( substr( $decrypteddata, 1, 1 ) );
+                                                if (length($decrypteddata) >= 4 + $addr_len )
+                                                {
+                                                    $dest_addr = substr( $decrypteddata, 2,$addr_len );
+                                                    $dest_port = unpack('n',substr($decrypteddata,2 + $addr_len,2));
+                                                    $decrypteddata = substr( $decrypteddata,4 + $addr_len );
+                                                }
+                                                else 
+                                                {
+                                                    return;
+                                                }
+                                            }
+                                            else 
+                                            {
+                                                return;
+                                            }
+                                        }
+                                        elsif ( $addrtype == 4 ) 
+                                        {
+                                            if ( length($decrypteddata) >= 19 ) 
+                                            {
+                                                $dest_addr = format_address(substr( $decrypteddata, 1, 16 ) );
+                                                $dest_port = unpack( "n", substr( $decrypteddata, 17, 2 ) );
+                                                $decrypteddata = substr( $decrypteddata, 19 );
+                                            }
+                                            else 
+                                            {
+                                                return;
+                                            }
                                         }
-                                    }
-                                    else 
-                                    {
-                                        last;
-                                    }
-                                }
-                                elsif ( $addrtype == 4 ) 
-                                {
-                                    if ( length($decrypteddata) >= 19 ) 
-                                    {
-                                        $dest_addr = format_address(substr( $decrypteddata, 1, 16 ) );
-                                        $dest_port = unpack( "n", substr( $decrypteddata, 17, 2 ) );
-                                        $decrypteddata = substr( $decrypteddata, 19 );
-                                    }
-                                    else 
-                                    {
-                                        last;
-                                    }
-                                }
-
-                                if ( $dest_addr eq '' ) 
-                                {
-                                    last;
-                                }
 =for comment
                                 AnyEvent::Socket::resolve_sockaddr($dest_addr, $dest_port, "tcp", 0, undef, sub 
                                 {
                                     my ( $ip_addr ) = @_;
                                     AE::log  info => "resolved " . AnyEvent::Socket::format_address( $ip_addr );
 =cut
-                                    $remotehandler = AnyEvent::Handle->new
-                                    (
-                                        autocork   => 1,
-                                        keepalive  => 1,
-                                        no_delay   => 1,
-                                        connect    => [ $dest_addr, $dest_port ],
-                                        on_connect => sub 
-                                        {
-                                        my ($rhandle,  $peerhost,$peerport, $retry) = @_;
-
-                                        unless ($rhandle) 
-                                        {
-                                            carp "couldn' t connect : $! ";
-                                            return;
-                                        }
-                                    },
-                                        on_eof => sub 
-                                        {
-                                        my $rhandler = shift;
-                                        AE::log info => " Remote : Done . ";
-                                        $mode = 0;
-                                        $rhandler->destroy();
-                                    },
-                                        on_error => sub 
-                                        {
-                                        my ( $rhandler, $fatal, $msg ) = @_;
-                                        AE::log error => $msg;
-                                        $mode = 0;
-                                        $rhandler->destroy();
-                                    },
-                                        on_prepare => sub
-                                        {
-                                            my $phandle = shift;
-                                            setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
-                                            },
-                                        on_read => sub 
-                                        {
-                                        my $remote_buffer = $remotehandler->rbuf();
-                                        my $plaindata = $remote_buffer;
-                                        my $encrypteddata;
-
-                                            if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
+                                        $remotehandler = AnyEvent::Handle->new
+                                        (
+                                            autocork   => 1,
+                                            keepalive  => 1,
+                                            no_delay   => 1,
+                                            connect    => [ $dest_addr, $dest_port ],
+                                            on_connect => sub 
                                             {
-                                                $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv); 
+                                                my ($rhandle,  $peerhost,$peerport, $retry) = @_;
+                                                AE::log info => "Connected with $peerhost : $peerport."; 
+                                            },
+                                            on_connect_error => sub
+                                            {
+                                                my ($chandle,  $msg) = @_;
+                                                AE::log error => $msg;
+                                                $chandle->destroy();
+                                                $mode = 0;
+                                            },
+                                            on_eof => sub 
+                                            {
+                                                my $rhandler = shift;
+                                                #AE::log info => " Remote : Done . ";
+                                                $rhandler->destroy();
+                                                $mode = 0;
+                                            },
+                                            on_error => sub 
+                                            {
+                                                my ( $rhandler, $fatal, $msg ) = @_;
+                                                AE::log error => $msg;
+                                                $rhandler->destroy();
+                                                $mode = 0;
+                                            },
+                                            on_prepare => sub
+                                            {
+                                                my $phandle = shift;
+                                                setsockopt($phandle->{fh}, SOL_SOCKET, SO_REUSEPORT, 1);
+                                            },
+                                            on_read => sub 
+                                            {
+                                                my $remote_buffer = $remotehandler->rbuf();
+                                                my $plaindata = $remote_buffer;
+                                                my $encrypteddata;
+
+                                                if ($self->{method} eq 'rc6' or $self->{method} =~ /cfb$/ or $self->{method} =~ /ctr$/ or $self->{method} =~ /^camellia/)
+                                                {
+                                                    $encrypteddata = $encryptor->encrypt($plaindata,$key,$iv); 
                                                 }
 
-                                            elsif($self->{method} eq 'chacha20-ietf')
-                                                    {
+                                                elsif($self->{method} eq 'chacha20-ietf')
+                                                {
                                                         #$encrypteddata = $encryptor->process($plaindata);
-                                                        $encrypteddata = $encryptor->chacha20_ietf_xor($plaindata,$iv,$key);
-                                                        }
+                                                    $encrypteddata = $encryptor->chacha20_ietf_xor($plaindata,$iv,$key);
+                                                }
                                                 else 
-                                            {
-                                                $encrypteddata = Mcrypt::mcrypt_encrypt($encryptor, $plaindata );
-                                            }
+                                                {
+                                                    $encrypteddata = Mcrypt::mcrypt_encrypt($encryptor, $plaindata );
+                                                }
                                         
-                                        my $datatosend;
-                                        if ( $mode == 0 ) 
-                                        {
-                                            $datatosend = $iv . $encrypteddata;
-                                            $mode       = 1;
-                                        }
-                                        else 
-                                        {
-                                            $datatosend = $encrypteddata;
-                                        }
-                                        $clienthandler->push_write($datatosend);
-                                        $remotehandler->{rbuf} = '';
-                                    }
-                                   );
-#                                }
+                                                my $datatosend;
+                                                if ( $mode == 0 ) 
+                                                {
+                                                    $datatosend = $iv . $encrypteddata;
+                                                    $mode       = 1;
+                                                }
+                                                else 
+                                                {
+                                                    $datatosend = $encrypteddata;
+                                                }
+                                                $clienthandler->push_write($datatosend);
+                                                $remotehandler->{rbuf} = '';
+                                            }
+                                       );
+                                  }
 #                                );
                             }
                             else 
@@ -329,7 +330,7 @@ Net::Shadowsocks::Server
 
 =head1 VERSION
 
-Version 0.8.1
+Version 0.8.2
 
 =head1 SYNOPSIS
 
